@@ -1,11 +1,11 @@
 ;; C code editing commands for Emacs
-;; Copyright (C) 1985, 1986, 1987 Free Software Foundation, Inc.
+;; Copyright (C) 1985, 1986, 1987, 1992 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 1, or (at your option)
+;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
@@ -273,6 +273,8 @@ is filled, preserving the comment indentation or line-starting decorations."
 	(save-excursion
 	  (goto-char insertpos)
 	  (self-insert-command (prefix-numeric-value arg)))
+      (if (> last-command-char 127) ; simply incredible kludge
+	  (setq last-command-char (- last-command-char 128)))
       (self-insert-command (prefix-numeric-value arg)))))
 
 (defun c-insert-brackets ()
