@@ -18,38 +18,6 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 
-#define Ctl(c) ((c)&037)
-
-/* Define the names of keymaps, just so people can refer to
-   them in calls to initial_define_key */
-
-extern struct Lisp_Keymap *global_map;
-				/* default global key bindings */
-
-extern struct Lisp_Keymap *meta_map;
-				/* The keymap used for globally bound
-				   ESC-prefixed default commands */
-
-extern struct Lisp_Keymap *control_x_map;
-				/* The keymap used for globally bound
-				   C-x-prefixed default commands */
-
-extern Lisp_Object Vminibuffer_local_map;
-				/* The keymap used by the minibuf for
-				   local bindings when spaces are allowed
-				   in the minibuf */
-
-extern Lisp_Object Vminibuffer_local_ns_map;
-				/* The keymap used by the minibuf for
-				   local bindings when spaces are not
-				   encouraged in the minibuf */
-
-/* keymap used for minibuffers when doing completion */
-extern Lisp_Object Vminibuffer_local_completion_map;
-
-/* keymap used for minibuffers when doing completion and require a match */
-extern Lisp_Object Vminibuffer_local_must_match_map;
-
 /* Previous command symbol found here for comparison */
 extern Lisp_Object last_command;
 
@@ -62,10 +30,27 @@ extern Lisp_Object Vexecuting_macro;
 #define INTERACTIVE (NILP (Vexecuting_macro) && !noninteractive)
 
 /* Set this nonzero to force reconsideration of mode line. */
-
 extern int redraw_mode_line;
 
 /* Nonzero means reading single-character input with prompt
    so put cursor on minibuffer after the prompt.  */
-
 extern int cursor_in_echo_area;
+
+extern Lisp_Object call_command_loop (Lisp_Object dummy);
+
+extern Lisp_Object Vkeyboard_translate_table;
+extern Lisp_Object Vthis_command;
+extern Lisp_Object Vlast_command;
+extern Lisp_Object Vunread_command_event;
+extern Lisp_Object Vlast_command_event;
+extern Lisp_Object Vlast_input_event;
+/* These two for compatibility; they are V... because they can be nil. */
+extern Lisp_Object Vlast_command_char;
+extern Lisp_Object Vlast_input_char;
+extern Lisp_Object Vlast_input_time;
+extern Lisp_Object Vcurrent_mouse_event;
+
+extern int zmacs_regions;
+extern int zmacs_region_active_p;
+extern int zmacs_region_stays;
+extern void zmacs_update_region (void);
