@@ -3,6 +3,11 @@
 
 #include "lwlib.h"
 
+/*
+extern char *strdup (const char *);
+extern int strcasecmp (const char *, const char *);
+*/
+
 typedef struct _widget_instance
 {
   Widget		widget;
@@ -16,7 +21,7 @@ typedef struct _widget_info
 {
   char*			type;
   char*			name;
-  BITS32		id;
+  LWLIB_ID		id;
   widget_value*		val;
   Boolean		busy;
   lw_callback		pre_activate_cb;
@@ -35,9 +40,15 @@ typedef struct _widget_creation_entry
   widget_creation_function	function;
 } widget_creation_entry;
 
+/* update all other instances of a widget.  Can be used in a callback when
+   a wiget has been used by the user */
 void
 lw_internal_update_other_instances (Widget widget, XtPointer closure,
 				    XtPointer call_data);
 
+/* get the widget_value for a widget in a given instance */
+widget_value*
+lw_get_widget_value_for_widget (widget_instance* instance, Widget w);
+
 #endif /* LWLIB_INTERNAL_H */
 
