@@ -1,11 +1,11 @@
-/* Definitions file for GNU Emacs running on Silicon Graphics 3.5
-   Copyright (C) 1987 Free Software Foundation, Inc.
+/* Definitions file for GNU Emacs running on Silicon Graphics system 3.6.
+   Copyright (C) 1987, 1992 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 1, or (at your option)
+the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -54,8 +54,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #define HAVE_TERMIO
 
-#define HAVE_DREM
-
 /*
  *	Define HAVE_TIMEVAL if the system supports the BSD style clock values.
  *	Look in <sys/time.h> for a timeval structure.
@@ -143,6 +141,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
  */
 
 #define sigsetmask(mask)	/* Null expansion */
+#define sigblock(x) x
 
 /* The IRIS defines SIGIO in signal.h, but doesn't implement it. */
 #undef SIGIO
@@ -189,10 +188,8 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #define NEED_ERRNO
 
-/* The symbol FIONREAD is defined, but the feature does not work.  */
-
-#define BROKEN_FIONREAD
-
 /* This is how to get the device name of the tty end of a pty.  */
 #define PTY_TTY_NAME_SPRINTF \
  	    sprintf (ptyname, "/dev/ttyq%d", minor (stb.st_rdev));
+
+#define FLOAT_CATCH_SIGILL
