@@ -1,11 +1,11 @@
 /* Window definitions for GNU Emacs.
-   Copyright (C) 1985, 1986 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 1992 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 1, or (at your option)
+the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -187,11 +187,6 @@ extern Lisp_Object Vwindow_system;
 /* Version number of X windows: 10, 11 or nil.  */
 extern Lisp_Object Vwindow_system_version;
 
-extern Lisp_Object Fnext_window ();
-extern Lisp_Object Fselect_window ();
-extern Lisp_Object Fdisplay_buffer ();
-extern Lisp_Object Fset_window_buffer ();
-
 /* Prompt to display in front of the minibuffer contents or nil */
 extern Lisp_Object Vminibuf_prompt;
 
